import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsString,
  IsBoolean,
  IsDate,
  IsInt,
  IsEnum,
  ValidateNested,
  IsLongitude,
  IsLatitude,
  ValidateIf,
  IsArray,
  ArrayMinSize,
} from 'class-validator';
import { Frequency } from '../entities/frequency.enum';
import { Transform, Type } from 'class-transformer';
import { mappingKeyToFrequency } from './utils/frequency.mapping';
import { MarginDTO } from './create.margin.dto';
import { ScheduleDTO } from './create.schedule.dto';

// TODO mapping schedule and margin (proto object  to margin and schedule fields )
export class CreateAdRequest {
  @IsString()
  @AutoMap()
  userUuid: string;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver?: boolean;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger?: boolean;

  @Transform(({ value }) => mappingKeyToFrequency(value), {
    toClassOnly: true,
  })
  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @ValidateIf((ad) => ad.frequency === 'PUNCTUAL')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  departure?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  fromDate?: Date;

  @ValidateIf((ad) => ad.frequency === 'RECURRENT')
  @Type(() => Date)
  @IsDate()
  @AutoMap()
  toDate?: Date;

  @ValidateNested({ each: true })
  @Type(() => ScheduleDTO)
  @AutoMap(() => ScheduleDTO)
  schedule: ScheduleDTO;

  @ValidateNested({ each: true })
  @Type(() => MarginDTO)
  @AutoMap(() => MarginDTO)
  marginDurations: MarginDTO;

  @ValidateIf((ad) => ad.driver)
  @IsInt()
  @AutoMap()
  seatsDriver?: number;

  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsPassenger?: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  strict?: boolean;

  @IsArray()
  @ArrayMinSize(2)
  @ValidateNested({ each: true })
  @Type(() => AddressDTO)
  @AutoMap(() => [AddressDTO])
  addresses: AddressDTO[];
}

class AddressDTO {
  @IsInt()
  @AutoMap()
  position: number;

  @IsLongitude()
  @AutoMap()
  lon: number;

  @IsLatitude()
  @AutoMap()
  lat: number;

  @IsString()
  @IsOptional()
  @AutoMap()
  name?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  houseNumber?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  street?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  locality?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  postalCode?: string;

  @IsString()
  @AutoMap()
  country: string;
}
